==> charmap_1.0.0.tar.gz: charmap-1.0.0/charmap.c <==
#include <hildon/hildon.h>
#include <gucharmap/gucharmap.h>

#include "maemo-gucharmap-chartable.h"

#define APP_NAME    "Character Map"
#define APP_VERSION "0.1.0"

typedef enum
{
  CHARMAP_DIALOG_SORT_BY_SCRIPT,
  CHARMAP_DIALOG_SORT_BY_CODE_BLOCK
} CharmapDialogSortMode;

static GtkButton *btn_sort_mode = NULL;
static GtkButton *btn_chapter   = NULL;

static GtkLabel *lbl_character   = NULL;
static GtkLabel *lbl_unicode     = NULL;
static GtkLabel *lbl_description = NULL;

static GtkEditable *edit_result = NULL;
static int sel_start, sel_end;

static GtkButton *btn_copy  = NULL;
static GtkButton *btn_clear = NULL;

static MaemoGucharmapChartable *chart = NULL;
static gboolean chart_button_pressed = FALSE;

static GtkLabel *make_label (gint width)
{
  GtkLabel *lbl = g_object_new(GTK_TYPE_LABEL, "visible", TRUE, NULL);
  gtk_widget_set_size_request(GTK_WIDGET(lbl), width, -1);
  gtk_label_set_line_wrap(lbl, TRUE);
  gtk_label_set_line_wrap_mode(lbl, PANGO_WRAP_WORD_CHAR);
  gtk_label_set_use_markup(lbl, TRUE);
  return lbl;
}

static void sort_mode_changed (HildonPickerButton *btn, GtkWidget *wnd)
{
  GucharmapChaptersModel *model = 
    (CHARMAP_DIALOG_SORT_BY_SCRIPT == hildon_picker_button_get_active(btn))
      ? gucharmap_script_chapters_model_new()
      : gucharmap_block_chapters_model_new();
  hildon_touch_selector_set_model(hildon_picker_button_get_selector(HILDON_PICKER_BUTTON(btn_chapter)), 0, GTK_TREE_MODEL(model));

  HildonTouchSelector *sel = hildon_picker_button_get_selector(HILDON_PICKER_BUTTON(btn_chapter));
  GtkTreeIter itr;
  gunichar new_char = g_utf8_get_char_validated("", -1);

  if (gucharmap_chapters_model_character_to_iter(model, new_char, &itr))
  {
    hildon_touch_selector_select_iter(sel, 0, &itr, FALSE);
    gucharmap_chartable_set_codepoint_list(GUCHARMAP_CHARTABLE(chart),
        gucharmap_chapters_model_get_codepoint_list(model, &itr));
    gucharmap_chartable_set_active_character(GUCHARMAP_CHARTABLE(chart), new_char);
  }
}

static GtkButton *make_sort_picker (GtkWidget *wnd)
{
  HildonTouchSelector *sel = HILDON_TOUCH_SELECTOR(hildon_touch_selector_new_text());
  hildon_touch_selector_append_text(sel, "Script");
  hildon_touch_selector_append_text(sel, "Unicode Block");
  hildon_touch_selector_set_active(sel, 0, 0);

  GtkButton *btn = g_object_new(HILDON_TYPE_PICKER_BUTTON,
      "visible",        TRUE,
      "arrangement",    HILDON_BUTTON_ARRANGEMENT_VERTICAL,
      "size",           HILDON_SIZE_FINGER_HEIGHT,
      "style",          HILDON_BUTTON_STYLE_PICKER,
      "title",          "Sort Mode",
      "touch-selector", sel,
      NULL);
  g_signal_connect(G_OBJECT(btn), "value-changed", (GCallback)sort_mode_changed, wnd);
  hildon_picker_button_set_active(HILDON_PICKER_BUTTON(btn), CHARMAP_DIALOG_SORT_BY_CODE_BLOCK);

  gtk_widget_show(GTK_WIDGET(btn));
  return btn;
}

static void chapter_changed (HildonPickerButton *btn, GtkWidget *wnd)
{
  HildonTouchSelector *sel = hildon_picker_button_get_selector(HILDON_PICKER_BUTTON(btn));
  GtkTreeModel *tm = hildon_touch_selector_get_model(sel, 0);
  GucharmapChaptersModel *model = GUCHARMAP_CHAPTERS_MODEL(tm);
  GtkTreeIter itr;

  if (hildon_touch_selector_get_selected(sel, 0, &itr))
    gucharmap_chartable_set_codepoint_list(GUCHARMAP_CHARTABLE(chart),
        gucharmap_chapters_model_get_codepoint_list(model, &itr));
}

static GtkButton *make_chapter_picker (GtkWidget *wnd)
{
  HildonTouchSelector *sel = g_object_new(HILDON_TYPE_TOUCH_SELECTOR, NULL);
  GtkListStore *ls = gtk_list_store_new(1, G_TYPE_STRING);
  HildonTouchSelectorColumn *col = hildon_touch_selector_append_column(HILDON_TOUCH_SELECTOR(sel), GTK_TREE_MODEL(ls), gtk_cell_renderer_text_new(), "text", 1, NULL);
  g_object_set(col, "text-column", 1, NULL);
  g_object_unref(ls);

  GtkButton *btn = g_object_new(HILDON_TYPE_PICKER_BUTTON,
      "visible",        TRUE,
      "arrangement",    HILDON_BUTTON_ARRANGEMENT_VERTICAL,
      "size",           HILDON_SIZE_FINGER_HEIGHT,
      "style",          HILDON_BUTTON_STYLE_PICKER,
      "title",          "Chapter",
      "touch-selector", sel,
      NULL);
  g_signal_connect(G_OBJECT(btn), "value-changed", (GCallback)chapter_changed, wnd);

  gtk_widget_show(GTK_WIDGET(btn));
  return btn;
}

static void about_activated (GtkWidget * widget, GtkWindow * wnd)
{
  GtkLabel *lbl = make_label(-1);
  gtk_label_set_text(lbl,
      "A simple character map for Maemo. Version " APP_VERSION ".\n"
      "Source code based on scv-reader, a Maemo app to modify the on-screen keyboard layout.\n"
      "Copyright © 2010 Gabriel Schulhof (scv-reader)\n"
      "Copyright © 2014 Peter Pichler (this application)\n"
      "Released under GPL 3.");

  GtkBox *vbox = g_object_new(GTK_TYPE_VBOX, "visible", TRUE, NULL);
  gtk_box_pack_start(vbox, GTK_WIDGET(lbl), FALSE, FALSE, 5);

  GtkDialog *dlg = g_object_new(GTK_TYPE_DIALOG,
      "visible", TRUE,
      "modal",   TRUE,
      "title",   "About " APP_NAME,
      NULL);
  gtk_window_set_transient_for(GTK_WINDOW(dlg), GTK_WINDOW(wnd));
  gtk_container_add(GTK_CONTAINER(dlg->vbox), GTK_WIDGET(vbox));

  gtk_dialog_run(dlg);
  gtk_widget_destroy(GTK_WIDGET(dlg));
}

static void make_menu (GtkWidget *wnd)
{
  btn_chapter = make_chapter_picker(wnd);
  btn_sort_mode = make_sort_picker(wnd);

  GtkButton *btn_about = g_object_new(HILDON_TYPE_BUTTON,
      "visible", TRUE,
      "size",    HILDON_SIZE_FINGER_HEIGHT,
      "title",   "About",
      NULL);
  g_signal_connect(G_OBJECT(btn_about), "clicked", G_CALLBACK(about_activated), wnd);

  GtkWidget *main_menu = g_object_new(HILDON_TYPE_APP_MENU, NULL);
  hildon_app_menu_append(HILDON_APP_MENU(main_menu), btn_sort_mode);
  hildon_app_menu_append(HILDON_APP_MENU(main_menu), btn_chapter);
  hildon_app_menu_append(HILDON_APP_MENU(main_menu), btn_about);
  hildon_window_set_app_menu(HILDON_WINDOW(wnd), HILDON_APP_MENU(main_menu));
}

static gboolean wnd_delete_event (GtkWidget *charmap, GdkEvent *event, gpointer null)
{
  gtk_main_quit();
  return FALSE;
}

static gboolean save_selection_info (GtkWidget *hpa, GdkEventButton *event, void *null)
{
  gtk_editable_get_selection_bounds(edit_result, &sel_start, &sel_end);
  chart_button_pressed = TRUE;
  return FALSE;
}

static void notify_active_character (GucharmapChartable *chart, GParamSpec *pspec, GtkWidget *ndw)
{
  const gunichar c = gucharmap_chartable_get_active_character(chart);
  const GUnicodeType type = gucharmap_unichar_type(c);

  GString *str = g_string_new("");
  g_string_append_unichar(str, c);
  char buffer[36];

  sprintf(buffer, "<span size=\"xx-large\">%s</span>",
     (  type != G_UNICODE_CONTROL
     && type != G_UNICODE_UNASSIGNED ) ? str->str : " ");
  gtk_label_set_markup(lbl_character, buffer);

  sprintf(buffer, "U+%04lX", (unsigned long)c);
  gtk_label_set_text(lbl_unicode, buffer);

  const gchar *description = gucharmap_get_unicode_name(c);
  gtk_label_set_text(lbl_description, description);

  if (chart_button_pressed)
  {
    if (type != G_UNICODE_CONTROL && type != G_UNICODE_UNASSIGNED)
    {
      if (sel_start != sel_end)
        gtk_editable_delete_text(edit_result, sel_start, sel_end);
      gtk_editable_insert_text(edit_result, str->str, -1, &sel_start);
      gtk_editable_set_position(edit_result, sel_end = sel_start);
    }
    chart_button_pressed = FALSE;
  }

  g_string_free(str, TRUE);
}

static void btn_copy_clicked (GtkWidget *btn, void *null)
{
  gtk_editable_select_region(edit_result, 0, -1);
  gtk_editable_copy_clipboard(edit_result);
}

static void btn_clear_clicked (GtkWidget *btn, void *null)
{
  gtk_editable_delete_text(edit_result, 0, -1);
}

static void make_wnd (void)
{
  GtkWidget *wnd = g_object_new(HILDON_TYPE_WINDOW, "title", APP_NAME, NULL);
  g_signal_connect(G_OBJECT(wnd), "delete-event", (GCallback)wnd_delete_event, NULL);

  chart = g_object_new(MAEMO_GUCHARMAP_CHARTABLE_TYPE, "visible", TRUE, NULL);
  g_signal_connect(G_OBJECT(chart), "notify::active-character", (GCallback)notify_active_character, wnd);

  GtkWidget *hpa = g_object_new(HILDON_TYPE_PANNABLE_AREA,
      "visible",        TRUE,
      "child",          chart,
      "hovershoot-max", 0,
      "vovershoot-max", 0,
      NULL);
  g_signal_connect(G_OBJECT(hpa), "button-press-event", (GCallback)save_selection_info, NULL);

  const gint w = 168;
  lbl_character = make_label(w);
  lbl_unicode = make_label(w);
  lbl_description = make_label(w);
  edit_result = g_object_new(HILDON_TYPE_ENTRY, "visible", TRUE, NULL);

  btn_copy = g_object_new(HILDON_TYPE_BUTTON,
      "visible", TRUE,
      "size",    HILDON_SIZE_FINGER_HEIGHT,
      "title",   "Copy",
      NULL);
  g_signal_connect(G_OBJECT(btn_copy), "clicked", (GCallback)btn_copy_clicked, NULL);

  btn_clear = g_object_new(HILDON_TYPE_BUTTON,
      "visible", TRUE,
      "size",    HILDON_SIZE_FINGER_HEIGHT,
      "title",   "Clear",
      NULL);
  g_signal_connect(G_OBJECT(btn_clear), "clicked", (GCallback)btn_clear_clicked, NULL);

  GtkBox *vbox = g_object_new(GTK_TYPE_VBOX, "visible", TRUE, NULL);
  gtk_box_pack_start(vbox, GTK_WIDGET(lbl_character),   FALSE, TRUE, 0);
  gtk_box_pack_start(vbox, GTK_WIDGET(lbl_unicode),     FALSE, TRUE, 0);
  gtk_box_pack_start(vbox, GTK_WIDGET(lbl_description), FALSE, TRUE, 0);
  gtk_box_pack_start(vbox, GTK_WIDGET(make_label(w)),   TRUE,  TRUE, 0);
  gtk_box_pack_start(vbox, GTK_WIDGET(edit_result),     FALSE, TRUE, 0);
  gtk_box_pack_start(vbox, GTK_WIDGET(btn_copy),        FALSE, TRUE, 0);
  gtk_box_pack_start(vbox, GTK_WIDGET(btn_clear),       FALSE, TRUE, 0);

  GtkBox *hbox = g_object_new(GTK_TYPE_HBOX, "visible", TRUE, NULL);
  gtk_box_pack_start(hbox, GTK_WIDGET(hpa),  TRUE,  TRUE, 0);
  gtk_box_pack_start(hbox, GTK_WIDGET(vbox), FALSE, TRUE, 0);
  gtk_widget_grab_focus(GTK_WIDGET(chart));

  gtk_container_add(GTK_CONTAINER(wnd), GTK_WIDGET(hbox));
  make_menu(wnd);
  gtk_widget_show(wnd);
}

int main (int argc, char** argv)
{
  gtk_init(&argc, &argv);
  make_wnd();
  gtk_main();
  return 0;
}

==> charmap_1.0.1.tar.gz: charmap-1.0.1/charmap.c <==
#include <hildon/hildon.h>
#include <gucharmap/gucharmap.h>

#include "maemo-gucharmap-chartable.h"

#define APP_NAME    "Character Map"
#define APP_VERSION "1.0.1"

typedef enum
{
  CHARMAP_DIALOG_SORT_BY_SCRIPT,
  CHARMAP_DIALOG_SORT_BY_CODE_BLOCK
} CharmapDialogSortMode;

static GtkButton *btn_sort_mode = NULL;
static GtkButton *btn_chapter   = NULL;

static GtkLabel *lbl_character   = NULL;
static GtkLabel *lbl_unicode     = NULL;
static GtkLabel *lbl_description = NULL;

static GtkEditable *edit_result = NULL;
static int sel_start, sel_end;

static GtkButton *btn_copy  = NULL;
static GtkButton *btn_clear = NULL;

static MaemoGucharmapChartable *chart = NULL;
static gboolean chart_button_pressed = FALSE;

static GtkLabel *make_label (gint width)
{
  GtkLabel *lbl = g_object_new(GTK_TYPE_LABEL, "visible", TRUE, NULL);
  gtk_widget_set_size_request(GTK_WIDGET(lbl), width, -1);
  gtk_label_set_line_wrap(lbl, TRUE);
  gtk_label_set_line_wrap_mode(lbl, PANGO_WRAP_WORD_CHAR);
  gtk_label_set_use_markup(lbl, TRUE);
  return lbl;
}

static void sort_mode_changed (HildonPickerButton *btn, GtkWidget *wnd)
{
  GucharmapChaptersModel *model = 
    (CHARMAP_DIALOG_SORT_BY_SCRIPT == hildon_picker_button_get_active(btn))
      ? gucharmap_script_chapters_model_new()
      : gucharmap_block_chapters_model_new();
  hildon_touch_selector_set_model(hildon_picker_button_get_selector(HILDON_PICKER_BUTTON(btn_chapter)), 0, GTK_TREE_MODEL(model));

  HildonTouchSelector *sel = hildon_picker_button_get_selector(HILDON_PICKER_BUTTON(btn_chapter));
  GtkTreeIter itr;
  gunichar new_char = g_utf8_get_char_validated("", -1);

  if (gucharmap_chapters_model_character_to_iter(model, new_char, &itr))
  {
    hildon_touch_selector_select_iter(sel, 0, &itr, FALSE);
    gucharmap_chartable_set_codepoint_list(GUCHARMAP_CHARTABLE(chart),
        gucharmap_chapters_model_get_codepoint_list(model, &itr));
    gucharmap_chartable_set_active_character(GUCHARMAP_CHARTABLE(chart), new_char);
  }
}

static GtkButton *make_sort_picker (GtkWidget *wnd)
{
  HildonTouchSelector *sel = HILDON_TOUCH_SELECTOR(hildon_touch_selector_new_text());
  hildon_touch_selector_append_text(sel, "Script");
  hildon_touch_selector_append_text(sel, "Unicode Block");
  hildon_touch_selector_set_active(sel, 0, 0);

  GtkButton *btn = g_object_new(HILDON_TYPE_PICKER_BUTTON,
      "visible",        TRUE,
      "arrangement",    HILDON_BUTTON_ARRANGEMENT_VERTICAL,
      "size",           HILDON_SIZE_FINGER_HEIGHT,
      "style",          HILDON_BUTTON_STYLE_PICKER,
      "title",          "Sort Mode",
      "touch-selector", sel,
      NULL);
  g_signal_connect(G_OBJECT(btn), "value-changed", (GCallback)sort_mode_changed, wnd);
  hildon_picker_button_set_active(HILDON_PICKER_BUTTON(btn), CHARMAP_DIALOG_SORT_BY_CODE_BLOCK);

  gtk_widget_show(GTK_WIDGET(btn));
  return btn;
}

static void chapter_changed (HildonPickerButton *btn, GtkWidget *wnd)
{
  HildonTouchSelector *sel = hildon_picker_button_get_selector(HILDON_PICKER_BUTTON(btn));
  GtkTreeModel *tm = hildon_touch_selector_get_model(sel, 0);
  GucharmapChaptersModel *model = GUCHARMAP_CHAPTERS_MODEL(tm);
  GtkTreeIter itr;

  if (hildon_touch_selector_get_selected(sel, 0, &itr))
    gucharmap_chartable_set_codepoint_list(GUCHARMAP_CHARTABLE(chart),
        gucharmap_chapters_model_get_codepoint_list(model, &itr));
}

static GtkButton *make_chapter_picker (GtkWidget *wnd)
{
  HildonTouchSelector *sel = g_object_new(HILDON_TYPE_TOUCH_SELECTOR, NULL);
  GtkListStore *ls = gtk_list_store_new(1, G_TYPE_STRING);
  HildonTouchSelectorColumn *col = hildon_touch_selector_append_column(HILDON_TOUCH_SELECTOR(sel), GTK_TREE_MODEL(ls), gtk_cell_renderer_text_new(), "text", 1, NULL);
  g_object_set(col, "text-column", 1, NULL);
  g_object_unref(ls);

  GtkButton *btn = g_object_new(HILDON_TYPE_PICKER_BUTTON,
      "visible",        TRUE,
      "arrangement",    HILDON_BUTTON_ARRANGEMENT_VERTICAL,
      "size",           HILDON_SIZE_FINGER_HEIGHT,
      "style",          HILDON_BUTTON_STYLE_PICKER,
      "title",          "Chapter",
      "touch-selector", sel,
      NULL);
  g_signal_connect(G_OBJECT(btn), "value-changed", (GCallback)chapter_changed, wnd);

  gtk_widget_show(GTK_WIDGET(btn));
  return btn;
}

static void about_activated (GtkWidget * widget, GtkWindow * wnd)
{
  GtkLabel *lbl = make_label(-1);
  gtk_label_set_text(lbl,
      "A simple character map for Maemo. Version " APP_VERSION ".\n"
      "Copyright © 2014 Peter Pichler (with special thanks "
      "to Gabriel Schulhof, the author of scv-reader)\n"
      "Released under GPL 3.");

  GtkBox *vbox = g_object_new(GTK_TYPE_VBOX, "visible", TRUE, NULL);
  gtk_box_pack_start(vbox, GTK_WIDGET(lbl), FALSE, FALSE, 5);

  GtkDialog *dlg = g_object_new(GTK_TYPE_DIALOG,
      "visible", TRUE,
      "modal",   TRUE,
      "title",   "About " APP_NAME,
      NULL);
  gtk_window_set_transient_for(GTK_WINDOW(dlg), GTK_WINDOW(wnd));
  gtk_container_add(GTK_CONTAINER(dlg->vbox), GTK_WIDGET(vbox));

  gtk_dialog_run(dlg);
  gtk_widget_destroy(GTK_WIDGET(dlg));
}

static void make_menu (GtkWidget *wnd)
{
  btn_chapter = make_chapter_picker(wnd);
  btn_sort_mode = make_sort_picker(wnd);

  GtkButton *btn_about = g_object_new(HILDON_TYPE_BUTTON,
      "visible", TRUE,
      "size",    HILDON_SIZE_FINGER_HEIGHT,
      "title",   "About",
      NULL);
  g_signal_connect(G_OBJECT(btn_about), "clicked", G_CALLBACK(about_activated), wnd);

  GtkWidget *main_menu = g_object_new(HILDON_TYPE_APP_MENU, NULL);
  hildon_app_menu_append(HILDON_APP_MENU(main_menu), btn_sort_mode);
  hildon_app_menu_append(HILDON_APP_MENU(main_menu), btn_chapter);
  hildon_app_menu_append(HILDON_APP_MENU(main_menu), btn_about);
  hildon_window_set_app_menu(HILDON_WINDOW(wnd), HILDON_APP_MENU(main_menu));
}

static gboolean wnd_delete_event (GtkWidget *charmap, GdkEvent *event, gpointer null)
{
  gtk_main_quit();
  return FALSE;
}

static gboolean save_selection_info (GtkWidget *hpa, GdkEventButton *event, void *null)
{
  gtk_editable_get_selection_bounds(edit_result, &sel_start, &sel_end);
  chart_button_pressed = TRUE;
  return FALSE;
}

static void notify_active_character (GucharmapChartable *chart, GParamSpec *pspec, GtkWidget *ndw)
{
  const gunichar c = gucharmap_chartable_get_active_character(chart);
  const GUnicodeType type = gucharmap_unichar_type(c);

  GString *str = g_string_new("");
  g_string_append_unichar(str, c);
  char buffer[36];

  sprintf(buffer, "<span size=\"xx-large\">%s</span>",
     (  type == G_UNICODE_CONTROL
     || type == G_UNICODE_UNASSIGNED )
         ? " "
         : (str->str[0] == '&' && str->str[1] == 0)
             ? "&amp;"
             : str->str);
  gtk_label_set_markup(lbl_character, buffer);

  sprintf(buffer, "U+%04lX", (unsigned long)c);
  gtk_label_set_text(lbl_unicode, buffer);

  const gchar *description = gucharmap_get_unicode_name(c);
  gtk_label_set_text(lbl_description, description);

  if (chart_button_pressed)
  {
    if (type != G_UNICODE_CONTROL && type != G_UNICODE_UNASSIGNED)
    {
      if (sel_start != sel_end)
        gtk_editable_delete_text(edit_result, sel_start, sel_end);
      gtk_editable_insert_text(edit_result, str->str, -1, &sel_start);
      gtk_editable_set_position(edit_result, sel_end = sel_start);
    }
    chart_button_pressed = FALSE;
  }

  g_string_free(str, TRUE);
}

static void btn_copy_clicked (GtkWidget *btn, void *null)
{
  gtk_editable_select_region(edit_result, 0, -1);
  gtk_editable_copy_clipboard(edit_result);
}

static void btn_clear_clicked (GtkWidget *btn, void *null)
{
  gtk_editable_delete_text(edit_result, 0, -1);
}

static void make_wnd (void)
{
  GtkWidget *wnd = g_object_new(HILDON_TYPE_WINDOW, "title", APP_NAME, NULL);
  g_signal_connect(G_OBJECT(wnd), "delete-event", (GCallback)wnd_delete_event, NULL);

  chart = g_object_new(MAEMO_GUCHARMAP_CHARTABLE_TYPE, "visible", TRUE, NULL);
  g_signal_connect(G_OBJECT(chart), "notify::active-character", (GCallback)notify_active_character, wnd);

  GtkWidget *hpa = g_object_new(HILDON_TYPE_PANNABLE_AREA,
      "visible",        TRUE,
      "child",          chart,
      "hovershoot-max", 0,
      "vovershoot-max", 0,
      NULL);
  g_signal_connect(G_OBJECT(hpa), "button-press-event", (GCallback)save_selection_info, NULL);

  const gint w = 168;
  lbl_character = make_label(w);
  lbl_unicode = make_label(w);
  lbl_description = make_label(w);
  edit_result = g_object_new(HILDON_TYPE_ENTRY, "visible", TRUE, NULL);

  btn_copy = g_object_new(HILDON_TYPE_BUTTON,
      "visible", TRUE,
      "size",    HILDON_SIZE_FINGER_HEIGHT,
      "title",   "Copy",
      NULL);
  g_signal_connect(G_OBJECT(btn_copy), "clicked", (GCallback)btn_copy_clicked, NULL);

  btn_clear = g_object_new(HILDON_TYPE_BUTTON,
      "visible", TRUE,
      "size",    HILDON_SIZE_FINGER_HEIGHT,
      "title",   "Clear",
      NULL);
  g_signal_connect(G_OBJECT(btn_clear), "clicked", (GCallback)btn_clear_clicked, NULL);

  GtkBox *vbox = g_object_new(GTK_TYPE_VBOX, "visible", TRUE, NULL);
  gtk_box_pack_start(vbox, GTK_WIDGET(lbl_character),   FALSE, TRUE, 0);
  gtk_box_pack_start(vbox, GTK_WIDGET(lbl_unicode),     FALSE, TRUE, 0);
  gtk_box_pack_start(vbox, GTK_WIDGET(lbl_description), FALSE, TRUE, 0);
  gtk_box_pack_start(vbox, GTK_WIDGET(make_label(w)),   TRUE,  TRUE, 0);
  gtk_box_pack_start(vbox, GTK_WIDGET(edit_result),     FALSE, TRUE, 0);
  gtk_box_pack_start(vbox, GTK_WIDGET(btn_copy),        FALSE, TRUE, 0);
  gtk_box_pack_start(vbox, GTK_WIDGET(btn_clear),       FALSE, TRUE, 0);

  GtkBox *hbox = g_object_new(GTK_TYPE_HBOX, "visible", TRUE, NULL);
  gtk_box_pack_start(hbox, GTK_WIDGET(hpa),  TRUE,  TRUE, 0);
  gtk_box_pack_start(hbox, GTK_WIDGET(vbox), FALSE, TRUE, 0);
  gtk_widget_grab_focus(GTK_WIDGET(chart));

  gtk_container_add(GTK_CONTAINER(wnd), GTK_WIDGET(hbox));
  make_menu(wnd);
  gtk_widget_show(wnd);
}

int main (int argc, char** argv)
{
  gtk_init(&argc, &argv);
  make_wnd();
  gtk_main();
  return 0;
}
